#version 410 core

layout(location = 0) in vec3 vertPos;
layout(location = 1) in vec3 vertNor;
layout(location = 2) in vec2 vertTex;
layout(location = 3) in vec3 instanceInfo;

uniform mat4 V, P;

out vec3 vertex_pos;
out vec3 vertex_nor;
out vec2 vertex_tex;
out float scalar;

//******************************************************************************

mat4 createIdentityMat()
{
    mat4 mat;
    
    for (int i = 0; i < 4; i++)
        for (int j = 0; j < 4; j++)
            mat[i][j] = (i == j) ? 1 : 0;
    
    return mat;
}

// rotation about y-axis
mat4 createLongitudeRot(float theta)
{
    mat4 mat = createIdentityMat();
    
    mat[0][0] = cos(theta);
    mat[0][2] = -sin(theta);
    mat[2][0] = sin(theta);
    mat[2][2] = cos(theta);
    
    return mat;
}

// rotation about x-axis
mat4 createLatitudeRot(float theta)
{
    mat4 mat = createIdentityMat();
    
    mat[1][1] = cos(theta);
    mat[1][2] = -sin(theta);
    mat[2][1] = sin(theta);
    mat[2][2] = cos(theta);
    
    return mat;
}

void main()
{
    mat4 latitudeRot = createLatitudeRot(radians(instanceInfo.x));
    mat4 longitudeRot = createLongitudeRot(radians(instanceInfo.y));
    
    mat4 M = longitudeRot * latitudeRot;
    
    vec4 tpos = vec4(vertPos, 1);
    tpos.z *= instanceInfo.z / 2 + 1;
    tpos = M * tpos;
    gl_Position = P * V * tpos;
    
    vertex_pos = tpos.xyz;
    vertex_nor = vec3(M * vec4(vertNor, 0));
    vertex_tex = vertTex;
    scalar = instanceInfo.z;
}
